@@ -1,18 +1,11 @@
-import pregex.core.pre as _pre
-import pregex.core.exceptions as _ex
-from pregex.core.quantifiers import Exactly as _Exactly
-
-
 __doc__ = """
 All classes within this module "assert" something about the provided pattern
 without having to match any additional characters. For example, :class:`MatchAtStart`
 ensures that the provided pattern matches only when it is found at the start of the string,
 while :class:`NotFollowedBy` asserts that a match must not be followed by one or more
-specified patterns. Another thing you should keep in mind is that some of these assertions
-cannot be quantified, as attempting that will cause a ``CannotBeQuantifiedException`` exception
-to be thrown. The only exception to this is quantifier :class:`~pregex.quantifiers.Optional`,
-which can be applied to every assertion as it does not result in the pattern having to be
-repeated more than once.
+specified patterns. Another thing you should keep in mind is that many of these assertions
+cannot be repeated, as attempting that will cause a ``CannotBeRepeatedException`` exception
+to be thrown.
 
 Classes & methods
 -------------------------------------------
@@ -22,6 +15,11 @@ along with any possible methods they may possess.
 """
 
 
+import pregex.core.pre as _pre
+import pregex.core.exceptions as _ex
+from typing import Union as _Union
+
+
 class __Assertion(_pre.Pregex):
     '''
     Constitutes the base class for `__Anchor` and `__Lookaround` classes.
@@ -47,7 +45,7 @@ class __Anchor(__Assertion):
     :raises InvalidArgumentTypeException: Parameter ``pre`` is neither a ``Pregex`` instance \
         nor a string.
     '''
-    def __init__(self, pre: _pre.Pregex or str, transform):
+    def __init__(self, pre: _Union[_pre.Pregex, str], transform):
         '''
         Constitutes the base class for all `anchor` classes that are part of this module.
 
@@ -57,7 +55,7 @@ class __Anchor(__Assertion):
         :raises InvalidArgumentTypeException: Parameter ``pre`` is neither a ``Pregex`` instance \
             nor a string.
         '''
-        super().__init__(transform(__class__._to_pregex(pre)))
+        super().__init__(str(transform(__class__._to_pregex(pre))))
 
 
 class __PositiveLookaround(__Assertion):
@@ -69,7 +67,7 @@ class __PositiveLookaround(__Assertion):
     :param Pregex | str assertion: A Pregex instance or string representing the `assertion` pattern.
     :param (Pregex, Pregex => str) transform: A `transform` function for the provided patterns.
     '''
-    def __init__(self, match: _pre.Pregex or str, assertion: _pre.Pregex or str, transform):
+    def __init__(self, match: _Union[_pre.Pregex, str], assertion: _Union[_pre.Pregex, str], transform):
         '''
         Constitutes the base class for classes ``FollowedBy`` and ``PrecededBy`` \
             that are part of this module.
@@ -79,10 +77,7 @@ class __PositiveLookaround(__Assertion):
         :param (Pregex, Pregex => str) transform: A `transform` function for the provided patterns.
         '''
         match = __class__._to_pregex(match)
-        assertion = __class__._to_pregex(assertion)
-        result = match if assertion._get_type() == _pre._Type.Empty \
-             else transform(match, assertion)
-        super().__init__(str(result))
+        super().__init__(str(transform(match, assertion)))
 
 
 class __NegativeLookaround(__Assertion):
@@ -90,7 +85,7 @@ class __NegativeLookaround(__Assertion):
     Constitutes the base class for classes ``NotFollowedBy`` and ``NotPrecededBy`` \
         that are part of this module.
 
-    :param Pregex | str *pres: Two or more Pregex instances, the first of which always \
+    :param Pregex | str pres: Two or more Pregex instances, the first of which always \
         represents the `match` pattern, while the rest constitute `assertion` patterns.
     :param (tuple[Pregex | str] => str) transform: A `transform` function for the provided patterns.
 
@@ -98,12 +93,12 @@ class __NegativeLookaround(__Assertion):
     :raises EmptyNegativeAssertionException: The empty string is provided \
         as one of the assertion patterns.
     '''
-    def __init__(self, pres: tuple[_pre.Pregex or str], transform) -> _pre.Pregex:
+    def __init__(self, pres: tuple[_Union[_pre.Pregex, str]], transform) -> _pre.Pregex:
         '''
         Constitutes the base class for classes ``NotFollowedBy`` and ``NotPrecededBy`` \
             that are part of this module.
 
-        :param Pregex | str *pres: Two or more Pregex instances, the first of which always \
+        :param Pregex | str pres: Two or more Pregex instances, the first of which always \
             represents the `match` pattern, while the rest constitute `assertion` patterns.
         :param (tuple[Pregex | str] => str) transform: A `transform` function for the provided patterns.
 
@@ -116,11 +111,7 @@ class __NegativeLookaround(__Assertion):
             raise _ex.NotEnoughArgumentsException(message)
         result = __class__._to_pregex(pres[0])
         for pre in pres[1:]:
-            pre = __class__._to_pregex(pre)
-            if pre._get_type() == _pre._Type.Empty:
-                result = _pre.Empty()
-                raise _ex.EmptyNegativeAssertionException()
-            result = _pre.Pregex(transform(result, pre), escape=False)
+            result = transform(result, pre)
         super().__init__(str(result))
 
 
@@ -133,10 +124,10 @@ class MatchAtStart(__Anchor):
     :raises InvalidArgumentTypeException: Parameter ``pre`` is neither a \
         ``Pregex`` instance nor a string.
 
-    :note: Cannot have a repeating quantifier applied to it.
+    :note: The resulting pattern cannot have a repeating quantifier applied to it.
     '''
 
-    def __init__(self, pre: _pre.Pregex or str):
+    def __init__(self, pre: _Union[_pre.Pregex, str]):
         '''
         Matches the provided pattern only if said pattern is at the start of the string.
 
@@ -145,9 +136,9 @@ class MatchAtStart(__Anchor):
         :raises InvalidArgumentTypeException: Parameter ``pre`` is neither a \
             ``Pregex`` instance nor a string.
 
-        :note: Cannot have a repeating quantifier applied to it.
+        :note: The resulting pattern cannot have a repeating quantifier applied to it.
         '''
-        super().__init__(pre, lambda pre: str(pre._match_at_start()))
+        super().__init__(pre, lambda pre: pre.match_at_start())
 
 
 class MatchAtEnd(__Anchor):
@@ -159,10 +150,10 @@ class MatchAtEnd(__Anchor):
     :raises InvalidArgumentTypeException: Parameter ``pre`` is neither a \
         ``Pregex`` instance nor a string.
 
-    :note: Cannot have a repeating quantifier applied to it.
+    :note: The resulting pattern cannot have a repeating quantifier applied to it.
     '''
 
-    def __init__(self, pre: _pre.Pregex or str):
+    def __init__(self, pre: _Union[_pre.Pregex, str]):
         '''
         Matches the provided pattern only if said pattern is at the end of the string.
 
@@ -171,9 +162,9 @@ class MatchAtEnd(__Anchor):
         :raises InvalidArgumentTypeException: Parameter ``pre`` is neither a \
             ``Pregex`` instance nor a string.
 
-        :note: Cannot have a repeating quantifier applied to it.
+        :note: The resulting pattern cannot have a repeating quantifier applied to it.
         '''
-        super().__init__(pre, lambda pre: str(pre._match_at_end()))
+        super().__init__(pre, lambda pre: pre.match_at_end())
 
 
 class MatchAtLineStart(__Anchor):
@@ -186,11 +177,11 @@ class MatchAtLineStart(__Anchor):
         ``Pregex`` instance nor a string.
 
     :note:
-        - Cannot have a repeating quantifier applied to it.
+        - The resulting pattern cannot have a repeating quantifier applied to it.
         - Uses meta character ``^`` since the `MULTILINE` flag is considered on.
     '''
 
-    def __init__(self, pre: _pre.Pregex or str):
+    def __init__(self, pre: _Union[_pre.Pregex, str]):
         '''
         Matches the provided pattern only if said pattern is at the end of the string.
 
@@ -200,10 +191,10 @@ class MatchAtLineStart(__Anchor):
             ``Pregex`` instance nor a string.
 
         :note:
-            - Cannot have a repeating quantifier applied to it.
+            - The resulting pattern cannot have a repeating quantifier applied to it.
             - Uses meta character ``^`` since the `MULTILINE` flag is considered on.
         '''
-        super().__init__(pre, lambda pre: str(pre._match_at_line_start()))
+        super().__init__(pre, lambda pre: pre.match_at_line_start())
 
 
 class MatchAtLineEnd(__Anchor):
@@ -216,11 +207,11 @@ class MatchAtLineEnd(__Anchor):
         ``Pregex`` instance nor a string.
 
     :note:
-        - Cannot have a repeating quantifier applied to it.
+        - The resulting pattern cannot have a repeating quantifier applied to it.
         - Uses meta character ``$`` since the `MULTILINE` flag is considered on.
     '''
 
-    def __init__(self, pre: _pre.Pregex or str):
+    def __init__(self, pre: _Union[_pre.Pregex, str]):
         '''
         Matches the provided pattern only if said pattern is at the end of a line.
 
@@ -230,10 +221,10 @@ class MatchAtLineEnd(__Anchor):
             ``Pregex`` instance nor a string.
 
         :note:
-            - Cannot have a repeating quantifier applied to it.
+            - The resulting pattern cannot have a repeating quantifier applied to it.
             - Uses meta character ``$`` since the `MULTILINE` flag is considered on.
         '''
-        super().__init__(pre, lambda pre: str(pre._match_at_line_end()))
+        super().__init__(pre, lambda pre: pre.match_at_line_end())
 
 
 class WordBoundary(__Anchor):
@@ -247,7 +238,7 @@ class WordBoundary(__Anchor):
         Asserts that the position, at which an instance of this class is placed, \
         must constitute a word boundary.
         '''
-        super().__init__(_pre.Empty(), lambda pre: str(pre + _pre.Pregex("\\b", escape=False)))
+        super().__init__(_pre.Empty(), lambda pre: pre.concat(_pre.Pregex("\\b", escape=False)))
 
 
 class NonWordBoundary(__Anchor):
@@ -261,7 +252,7 @@ class NonWordBoundary(__Anchor):
         Asserts that the position, at which an instance of this class is placed, \
         must not constitute a word boundary.
         '''
-        super().__init__(_pre.Empty(), lambda pre: str(pre + _pre.Pregex("\\B", escape=False)))
+        super().__init__(_pre.Empty(), lambda pre: pre.concat(_pre.Pregex("\\B", escape=False)))
 
 
 class FollowedBy(__PositiveLookaround):
@@ -277,10 +268,10 @@ class FollowedBy(__PositiveLookaround):
     :raises InvalidArgumentTypeException: At least one of the provided arguments \
         is neither a ``Pregex`` instance nor a string.
 
-    :note: Cannot have a repeating quantifier applied to it.
+    :note: The resulting pattern cannot have a repeating quantifier applied to it.
     '''
 
-    def __init__(self, match: _pre.Pregex or str, assertion: _pre.Pregex or str):
+    def __init__(self, match: _Union[_pre.Pregex, str], assertion: _Union[_pre.Pregex, str]):
         '''
         Matches pattern ``match`` only if it is followed by pattern \
         ``assertion``, without the latter being included into the match.
@@ -293,9 +284,85 @@ class FollowedBy(__PositiveLookaround):
         :raises InvalidArgumentTypeException: At least one of the provided arguments \
             is neither a ``Pregex`` instance nor a string.
 
-        :note: Cannot have a repeating quantifier applied to it.
+        :note: The resulting pattern cannot have a repeating quantifier applied to it.
         '''
-        super().__init__(match, assertion, lambda pre1, pre2: str(pre1._followed_by(pre2)))
+        super().__init__(match, assertion, lambda pre1, pre2: pre1.followed_by(pre2))
+
+
+class PrecededBy(__PositiveLookaround):
+    '''
+    Matches pattern ``match`` only if it is preceded by pattern \
+    ``assertion``, without the latter being included into the match.
+
+    :param Pregex | str match: A Pregex instance or string \
+        representing the `match` pattern.
+    :param Pregex | str assertion: A Pregex instance or string \
+        representing the `assertion` pattern.
+
+    :raises InvalidArgumentTypeException: At least one of the provided arguments \
+        is neither a ``Pregex`` instance nor a string.
+    :raises NonFixedWidthPatternException: Parameter ``assertion`` \
+        corresponds to a pattern that does not have a fixed width.
+
+    :note: The resulting pattern cannot have a repeating quantifier applied to it.
+    '''
+
+    def __init__(self, match: _Union[_pre.Pregex, str], assertion: _Union[_pre.Pregex, str]):
+        '''
+        Matches pattern ``match`` only if it is preceded by pattern \
+        ``assertion``, without the latter being included into the match.
+
+        :param Pregex | str match: A Pregex instance or string \
+            representing the `match` pattern.
+        :param Pregex | str assertion: A Pregex instance or string \
+            representing the `assertion` pattern.
+
+        :raises InvalidArgumentTypeException: At least one of the provided arguments \
+            is neither a ``Pregex`` instance nor a string.
+        :raises NonFixedWidthPatternException: Parameter ``assertion`` \
+            corresponds to a pattern that does not have a fixed width.
+
+        :note: The resulting pattern cannot have a repeating quantifier applied to it.
+        '''
+        super().__init__(match, assertion, lambda pre1, pre2: pre1.preceded_by(pre2))
+
+
+class EnclosedBy(__PositiveLookaround):
+    '''
+    Matches pattern ``match`` only if it is both preceded and followed \
+    by pattern ``assertion``, without the latter being included into the match.
+
+    :param Pregex | str match: A Pregex instance or string \
+        representing the `match` pattern.
+    :param Pregex | str assertion: A Pregex instance or string \
+        representing the *assertion* pattern.
+
+    :raises InvalidArgumentTypeException: At least one of the provided arguments \
+        is neither a ``Pregex`` instance nor a string.
+    :raises NonFixedWidthPatternException: Parameter ``assertion`` \
+        corresponds to a pattern that does not have a fixed width.
+
+    :note: The resulting pattern cannot have a repeating quantifier applied to it.
+    '''
+
+    def __init__(self, match: _Union[_pre.Pregex, str], assertion: _Union[_pre.Pregex, str]):
+        '''
+        Matches pattern ``match`` only if it is both preceded and followed \
+        by pattern ``assertion``, without the latter being included into the match.
+
+        :param Pregex | str match: A Pregex instance or string \
+            representing the `match` pattern.
+        :param Pregex | str assertion: A Pregex instance or string \
+            representing the *assertion* pattern.
+
+        :raises InvalidArgumentTypeException: At least one of the provided arguments \
+            is neither a ``Pregex`` instance nor a string.
+        :raises NonFixedWidthPatternException: Parameter ``assertion`` \
+            corresponds to a pattern that does not have a fixed width.
+
+        :note: The resulting pattern cannot have a repeating quantifier applied to it.
+        '''
+        super().__init__(match, assertion, lambda pre1, pre2: pre1.enclosed_by(pre2))
 
 
 class NotFollowedBy(__NegativeLookaround):
@@ -314,7 +381,7 @@ class NotFollowedBy(__NegativeLookaround):
         patterns is the empty-string pattern.
     '''
 
-    def __init__(self, match: _pre.Pregex or str, *assertions: _pre.Pregex or str):
+    def __init__(self, match: _Union[_pre.Pregex, str], *assertions: _Union[_pre.Pregex, str]):
         '''
         Matches pattern ``match`` only if it is not followed by any one of \
         the provided ``assertion`` patterns.
@@ -330,49 +397,7 @@ class NotFollowedBy(__NegativeLookaround):
             patterns is the empty-string pattern.
         '''
         super().__init__((match, *assertions),
-            lambda pre1, pre2: str(pre1._not_followed_by(pre2)))
-
-
-class PrecededBy(__PositiveLookaround):
-    '''
-    Matches pattern ``match`` only if it is preceded by pattern \
-    ``assertion``, without the latter being included into the match.
-
-    :param Pregex | str match: A Pregex instance or string \
-        representing the `match` pattern.
-    :param Pregex | str assertion: A Pregex instance or string \
-        representing the `assertion` pattern.
-
-    :raises InvalidArgumentTypeException: At least one of the provided arguments \
-        is neither a ``Pregex`` instance nor a string.
-    :raises NonFixedWidthPatternException: A non-fixed-width pattern \
-        is provided in place of parameter ``assertion``.
-
-    :note: Cannot have a repeating quantifier applied to it.
-    '''
-
-    def __init__(self, match: _pre.Pregex or str, assertion: _pre.Pregex or str):
-        '''
-        Matches pattern ``match`` only if it is preceded by pattern \
-        ``assertion``, without the latter being included into the match.
-
-        :param Pregex | str match: A Pregex instance or string \
-            representing the `match` pattern.
-        :param Pregex | str assertion: A Pregex instance or string \
-            representing the `assertion` pattern.
-
-        :raises InvalidArgumentTypeException: At least one of the provided arguments \
-            is neither a ``Pregex`` instance nor a string.
-        :raises NonFixedWidthPatternException: A non-fixed-width pattern \
-            is provided in place of parameter ``assertion``.
-
-        :note: Cannot have a repeating quantifier applied to it.
-        '''
-        if isinstance(assertion, _pre.Pregex):
-            if assertion._get_type() == _pre._Type.Quantifier \
-                and (not isinstance(assertion, _Exactly)):
-                raise _ex.NonFixedWidthPatternException(self, assertion)
-        super().__init__(match, assertion, lambda pre1, pre2: str(pre1._preceded_by(pre2)))   
+            lambda pre1, pre2: pre1.not_followed_by(pre2))
 
 
 class NotPrecededBy(__NegativeLookaround):
@@ -390,10 +415,10 @@ class NotPrecededBy(__NegativeLookaround):
     :raises EmptyNegativeAssertionException: At least one of the provided assertion \
         patterns is the empty-string pattern.
     :raises NonFixedWidthPatternException: At least one of the provided assertion \
-        patterns is a non-fixed-width pattern.
+        patterns does not have a fixed width.
     '''
 
-    def __init__(self, match: _pre.Pregex or str, *assertions: _pre.Pregex or str):
+    def __init__(self, match: _Union[_pre.Pregex, str], *assertions: _Union[_pre.Pregex, str]):
         '''
         Matches pattern ``match`` only if it is not preceded by any one of \
         the provided ``assertion`` patterns.
@@ -408,60 +433,15 @@ class NotPrecededBy(__NegativeLookaround):
         :raises EmptyNegativeAssertionException: At least one of the provided assertion \
             patterns is the empty-string pattern.
         :raises NonFixedWidthPatternException: At least one of the provided assertion \
-            patterns is a non-fixed-width pattern.
+            patterns does not have a fixed width.
         '''
-        for pre in assertions:
-            if isinstance(pre, _pre.Pregex):
-                if pre._get_type() == _pre._Type.Quantifier and (not isinstance(pre, _Exactly)):
-                    raise _ex.NonFixedWidthPatternException(self, pre)
-        super().__init__((match, *assertions), lambda pre1, pre2: str(pre1._not_preceded_by(pre2)))
-
-
-class EnclosedBy(__PositiveLookaround):
-    '''
-    Matches pattern ``match`` only if it is both preceded and followed \
-    by pattern ``assertion``, without the latter being included into the match.
-
-    :param Pregex | str match: A Pregex instance or string \
-        representing the `match` pattern.
-    :param Pregex | str assertion: A Pregex instance or string \
-        representing the *assertion* pattern.
-
-    :raises InvalidArgumentTypeException: At least one of the provided arguments \
-        is neither a ``Pregex`` instance nor a string.
-    :raises NonFixedWidthPatternException: Parameter ``assertion`` \
-        corresponds to a non-fixed-width pattern.
-
-    :note: Cannot have a repeating quantifier applied to it.
-    '''
-
-    def __init__(self, match: _pre.Pregex or str, assertion: _pre.Pregex or str):
-        '''
-        Matches pattern ``match`` only if it is both preceded and followed \
-        by pattern ``assertion``, without the latter being included into the match.
-
-        :param Pregex | str match: A Pregex instance or string \
-            representing the `match` pattern.
-        :param Pregex | str assertion: A Pregex instance or string \
-            representing the *assertion* pattern.
-
-        :raises InvalidArgumentTypeException: At least one of the provided arguments \
-            is neither a ``Pregex`` instance nor a string.
-        :raises NonFixedWidthPatternException: Parameter ``assertion`` \
-            corresponds to a non-fixed-width pattern.
-
-        :note: Cannot have a repeating quantifier applied to it.
-        '''
-        if isinstance(assertion, _pre.Pregex):
-            if assertion._get_type() == _pre._Type.Quantifier \
-                and (not isinstance(assertion, _Exactly)):
-                raise _ex.NonFixedWidthPatternException(self, assertion)
-        super().__init__(match, assertion, lambda pre1, pre2: str(pre1._enclosed_by(pre2)))    
+        super().__init__((match, *assertions),
+            lambda pre1, pre2: pre1.not_preceded_by(pre2))
 
 
 class NotEnclosedBy(__NegativeLookaround):
     '''
-    Matches pattern ``match`` only if it neither preceded nor followed \
+    Matches pattern ``match`` only if it is neither preceded nor followed \
     by any one of the provided ``assertion`` patterns.
 
     :param Pregex | str match: The pattern that is to be matched.
@@ -471,15 +451,15 @@ class NotEnclosedBy(__NegativeLookaround):
     :raises NotEnoughArgumentsException: No assertion patterns were provided.
     :raises InvalidArgumentTypeException: At least one of the provided arguments \
         is neither a ``Pregex`` instance nor a string.
-    :raises EmptyNegativeAssertionException: Parameter ``assertion`` \
-        corresponds to the empty-string pattern.
-    :raises NonFixedWidthPatternException: Parameter ``assertion`` \
-        corresponds to a non-fixed-width pattern.
+    :raises EmptyNegativeAssertionException: At least one of the provided assertion \
+        patterns is the empty-string pattern.
+    :raises NonFixedWidthPatternException: At least one of the provided assertion \
+        patterns does not have a fixed width.
     '''
 
-    def __init__(self, match: _pre.Pregex or str, *assertions: _pre.Pregex or str):
+    def __init__(self, match: _Union[_pre.Pregex, str], *assertions: _Union[_pre.Pregex, str]):
         '''
-        Matches pattern ``match`` only if it neither preceded nor followed \
+        Matches pattern ``match`` only if it is neither preceded nor followed \
         by any one of the provided ``assertion`` patterns.
 
         :param Pregex | str match: The pattern that is to be matched.
@@ -489,13 +469,10 @@ class NotEnclosedBy(__NegativeLookaround):
         :raises NotEnoughArgumentsException: No assertion patterns were provided.
         :raises InvalidArgumentTypeException: At least one of the provided arguments \
             is neither a ``Pregex`` instance nor a string.
-        :raises EmptyNegativeAssertionException: Parameter ``assertion`` \
-            corresponds to the empty-string pattern.
-        :raises NonFixedWidthPatternException: Parameter ``assertion`` \
-            corresponds to a non-fixed-width pattern.
+        :raises EmptyNegativeAssertionException: At least one of the provided assertion \
+            patterns is the empty-string pattern.
+        :raises NonFixedWidthPatternException: At least one of the provided assertion \
+            patterns does not have a fixed width.
         '''
-        for pre in assertions:
-            if isinstance(pre, _pre.Pregex):
-                if pre._get_type() == _pre._Type.Quantifier and (not isinstance(pre, _Exactly)):
-                    raise _ex.NonFixedWidthPatternException(self, pre)
-        super().__init__((match, *assertions), lambda pre1, pre2: str(pre1._not_enclosed_by(pre2))) 
\ No newline at end of file
+        super().__init__((match, *assertions),
+            lambda pre1, pre2: pre1.not_enclosed_by(pre2))
\ No newline at end of file
